#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//-- Sampling frequency --//
#define	S48KHZ		0
#define	S32KHZ		1
#define	S16KHZ		2
#define	S8KHZ		3
#define	S44_1KHZ	4
#define	S22_05KHZ	5
#define	S11_025KHZ	6

//-- CH number --//
#define	CH16		0

//-- Sequence Difference Max --//
#define SEQUENCE_DIFF_MAX	0x100

//-- ich file version No. --//
#define FILE_VER			0x10000000

//-- Payload length(Frame size) --//
unsigned long payload_tbl[7]={
//   16ch 
	6144,   // 48k
	4096, 	// 32k
	2048, 	// 16k
	1024,   // 8k
	6144, 	// 44k
	3072, 	// 22k
	1536    // 11k
	};


//-- Sampling frequency teble --//
unsigned long sampfreq_tbl[7] = {
						48000,	// 48k
						32000,  // 32k
						16000,  // 16k
						 8000,  // 8k
						44100,  // 44k
						22050,  // 22k
						11025   // 11k
	};
		
int main(int argc, char* argv[]){
	
	FILE *ich_fp = NULL;
	FILE *log_fp = NULL;
	char ich_name[256] = "";
	char log_name[256] = "";
	unsigned long tmp_buf = 0;
	unsigned long smp_buf = 0;	//freq=48k, payload = 6144(default)
	unsigned long ch_buf = 0;	//ch = 16(default)
	unsigned long payload = 0;
	unsigned short payload_cur = 0;
	unsigned short payload_err_cnt = 0;
	unsigned short seq_num = 0;
	unsigned short seq_num_chk = 0;
	unsigned short seq_num_diff = 0;
	unsigned short i = 0;
	long file_size = 0;
	long now_fsize = 0;
	long pkt_total_num = 0;
	long err_cnt = 0;
	char err_flag = 0;
	
	printf("-----------------------------------------\n");
	printf("/*       ich file packet checker       */\n");
	printf("-----------------------------------------\n\n");

	if(argc == 1)	goto HELP_MESG;
	if(argc > 2)	goto HELP_MESG;

	strcpy(ich_name,argv[1]);

	ich_fp = fopen(ich_name,"rb");
	if(ich_fp == NULL){
		printf("*** Error : %s file isn't found !! ***\n",ich_name);
		goto END_PROC;
	}
	
	memcpy(log_name, ich_name, strlen(ich_name)-4);
	strcat(log_name, "chk.log");
	log_fp = fopen(log_name, "w");
	if(log_fp == NULL){
		printf("*** Error : %s file can not be opened!! ***\n", log_name);
		goto END_PROC;
	}
	fprintf(log_fp, "\t\t--------  %s packet check log file -------\n", log_name);

	//-- Check file version --//
	fseek(ich_fp, 0, SEEK_SET);	
	fread(&tmp_buf,sizeof(tmp_buf),1,ich_fp);
	if(tmp_buf != FILE_VER){
		printf("*** Error : This file isn't ich file format !! ***\n");
		goto END_PROC;
	}

	//-- Check sampling frequency --//
	fseek(ich_fp, 4, SEEK_SET);	
	fread(&smp_buf,sizeof(tmp_buf),1,ich_fp);
	
	//-- Check channel --//
	fseek(ich_fp, 8, SEEK_SET);	
	fread(&ch_buf,sizeof(ch_buf),1,ich_fp);

	if (ch_buf != 0x00000000){
		printf("ERROR: Invalid channels , not 16 channels file\n");
		goto END_PROC;
	}

	//-- Payload length --//
	payload = payload_tbl[smp_buf];
	
	//-- Display --//
	printf("Sampling Frequency   : %lu Hz\n",sampfreq_tbl[smp_buf]);
	printf("Channel number       : %d ch\n",16);
	printf("Payload length       : %lu Byte\n",payload);

        fprintf(log_fp, "Sampling Frequency   : %lu Hz\n",sampfreq_tbl[smp_buf]);
        fprintf(log_fp, "Channel number       : %d ch\n",16);
        fprintf(log_fp, "Payload length       : %lu Byte\n",payload);

	//-- Check file last pointer --//
	fseek(ich_fp,0,SEEK_END);
	file_size = ftell(ich_fp);
	printf("File size            : %lu Byte\n",file_size);

	//-- Shift header pointer --//
	fseek(ich_fp, 16, SEEK_SET);

	getchar();	
	//-- Check each packet --//
	while(1){
		
		pkt_total_num++;
		fread(&payload_cur, sizeof(unsigned short), 1, ich_fp);
		fread(&seq_num, sizeof(unsigned short), 1, ich_fp);
		
		if(payload_cur==payload){
			//-- Check sequence numbers --//
			if(seq_num != seq_num_chk){
				seq_num_diff = seq_num - seq_num_chk;
				err_cnt += seq_num_diff;

				if(seq_num_diff > SEQUENCE_DIFF_MAX){
					// lost so much packets
					err_flag = 1;
				}
			
				// print the lost packet sequence no
				for (i=seq_num_chk; i<seq_num; i++){
					fprintf(log_fp, "lost packet No: %u\n", i);
				}

				seq_num_chk += seq_num_diff;

			}//if...
		}else{
			payload_err_cnt++;
		}

		//-- Move to the next sequence number position
		fseek(ich_fp, payload, SEEK_CUR);

		//-- Check the end of file
		now_fsize = ftell(ich_fp);
		if(now_fsize >= file_size) {
			// file end 
			break;
		}

		seq_num_chk++;

	} //while
	
	
	printf("Packet total numbers :   %lu\n", pkt_total_num);
	printf("Payload header error count : %d\n", payload_err_cnt);

	fprintf(log_fp, "Packet total numbers : %lu\n\n", pkt_total_num);
	fprintf(log_fp, "Payload header error count : %d\n", payload_err_cnt);

	if(err_flag == 1){
		printf("This ICH file loses so much packets, therefore the ICH file is invalid !!\n");
		fprintf(log_fp, "This ICH file loses so much packets, therefore that is invalid !!\n");
	}
	else{
		printf("Lost packet          : %ld\n\n", err_cnt);
		fprintf(log_fp, "Lost packet            : %ld\n\n", err_cnt);
	}

	goto END_PROC;

HELP_MESG:

	printf("ichpkchk [xxxx.ich]\n");

END_PROC:

	if(ich_fp != NULL) fclose(ich_fp);
	if(log_fp != NULL) fclose(log_fp);
	return 0;
}


