/********************************************************/
/*   wavtooch.cpp - creates och files from wav files    */
/********************************************************/
#define _LINUX_

#include<stdlib.h>
#include<stdio.h>
#include<string.h>
#include<math.h>

#ifdef _LINUX_
#define _MAX_PATH	512
#define _MAX_FNAME	512
#define _MAX_EXT	512
#define _MAX_DRIVE	32
#define _MAX_DIR	512
#endif

#define MAXSAMPLE	192				// maximum number of samples

#define MAXCHANNEL	16				// maximum number of channels
#define MAXNSEQ		65536				// maximum number of packet
#define FDNUM		64				// maximum number of file desciptors
#define OCHCHAN		4				// och channels
#define WAVCH		1					

void OpenWavName(unsigned short ctr);						// open wave files for reading
void createOchName(unsigned short ctr);						// create och filename
void Writeheader(unsigned short ctr);						// write och header
void rd_wav_payload(unsigned short ctr, unsigned short nsamp);			// read wave file packet
void transpose_pkt(unsigned short nsamp, unsigned short ochch_ctr);		// copy to temporary array
void wr_audio(unsigned short och_ctr, unsigned short nsamp);			// write data
void HelpScreen();

char *f_ptr[FDNUM];								// pointer to filename (och files)				
char *fw_ptr[FDNUM];								// pointer to filename (wave files)
char *modeptr;

char ochname[FDNUM][_MAX_PATH];							// och filename array
char name [_MAX_FNAME] = "wv2och";
char filecnt[3];
char suffix [_MAX_EXT] = ".och";

char ochmask[_MAX_FNAME];
char firstwav[_MAX_PATH];

char wavname[FDNUM][_MAX_PATH];							// wave filename array
char wavpathname[_MAX_PATH];
char wavtoochdir[_MAX_PATH];
char wname [_MAX_FNAME] = "wavech";
char wavcnt[3];
char wavsuff[_MAX_EXT] = ".wav";

unsigned short i;
unsigned short ctr=0, ochch_ctr=0, och_ctr=0, nread, nread2;

unsigned short fctr_och, fctr_wav;				// number of och files, number of wave files	
unsigned short nsamp /*mctr*/;					// number of samples


unsigned int version;
unsigned int rate;						// sampling frequency
unsigned int hexrate;						// sampling frequency (hex)
unsigned int wvdatsize;				
unsigned short wav_pktsize;
unsigned short och_pktsize;
unsigned short och_pktsize_resv;				// och payload length (reserved)
unsigned short plnum;						// number of payloads
unsigned short plnum_resv;					// number of payloads (reserved)
unsigned int resv;						// reserved (reserved portion of och file)

unsigned short tr_data[OCHCHAN][MAXSAMPLE];			// temporary array
unsigned short wav_table[MAXSAMPLE];				// array to handle read wav file


FILE *mult_stream[FDNUM];					// file descriptors ( for och files )
FILE *wav_stream[FDNUM];					// file descriptors ( for wav files )
FILE *headerstrm;						// file descriptors	( for writing header )

int neg;
int indicator;

int ext_serch(char *ichfile){
                                                                                                                                               
        int st=0;
        int cnt;
                                                                                                                                               
        for( cnt=0 ; ; cnt++ ){
                if(*(ichfile+cnt)=='.') break;
        }
        if(*(ichfile+cnt+0)!='.') st=-1;
        if(*(ichfile+cnt+1)!='i') st=-1;
        if(*(ichfile+cnt+2)!='c') st=-1;
        if(*(ichfile+cnt+3)!='h') st=-1;
                                                                                                                                               
        return st;
}

void itoa( int ctr, char *wavcnt, int mode ){

	sprintf(wavcnt, "%d", ctr);

}

//=======================================================
//-- main--
//=======================================================
int main(int argc, char **argv)
{
	if(argc < 2)
	{
		printf("Too few parameters.\n");
		HelpScreen();
		exit(1);
	}
	else if(argc > 2)
	{
		printf("Too many parameters.\n");
		HelpScreen();
		exit(1);
	}
	else if(argc == 1)
	{
		HelpScreen();
		exit(1);
	}

	// -- 16 channel -> 16 wav
	fctr_wav = 16;

	*argv++;
	strcpy(wavpathname, *argv);

	strcpy(firstwav, wavpathname);
	strcat(firstwav, "/");
	strcat(firstwav, "wavech0.wav");

	printf("Firstwav : %s\n", firstwav);			
		
	if((headerstrm = fopen(firstwav, "rb" )) != NULL )				// must set the name of first file to read
 	{										// read wavch0.wav sampling rate and data size
	    fseek(headerstrm, 24, SEEK_SET);
	    fread(&rate, sizeof(unsigned int), 1, headerstrm);

	    fseek(headerstrm, 40, SEEK_SET);
	    fread(&wvdatsize, sizeof(unsigned int), 1, headerstrm);

	    fclose(headerstrm);

 	}else
 	{
	    printf( "Could not find or open required WAV file in specified path.\n" );
	    exit(1);
  	}

	//equivalent number of samples according to sampling rate setting
		switch( rate )
		{
			case 48000:
				nsamp = 192;
				hexrate = 0x0000;
				break;

			case 32000:
				nsamp = 128;
				hexrate = 0x0001;
				break;

			case 16000:
				nsamp = 64;
				hexrate = 0x0002;
				break;
        
			case 8000:
				nsamp = 32;
				hexrate = 0x0003;
				break;
       
			case 44100:
				nsamp = 192;
				hexrate = 0x0004;
				break;
        
			case 22050:
				nsamp = 96;
				hexrate = 0x0005;
				break;
        
			case 11025:
				nsamp = 48;
				hexrate = 0x0006;
				break;
		}

	/******************************** calculation of sizes ************************************************/

	fctr_och = fctr_wav/OCHCHAN;
	wav_pktsize = (2*WAVCH*nsamp);
	och_pktsize = (2*OCHCHAN*nsamp);
	plnum = (wvdatsize/wav_pktsize);

	/******************************** check calculated values ************************************************/
//	printf("\n");
//	printf("sampling rate: %d\n", rate);
//	printf("wave data size: %d\n",wvdatsize); 
//	printf("och files to be created: %d\n", fctr_och);
//	printf("wave packet size: %d\n", wav_pktsize);
//	printf("och packet size: %d\n", och_pktsize);
//	printf("number of packets: %d\n\n", plnum);

	/*************************************************************************************************/

    for(ctr=0; ctr<fctr_wav; ctr++)									
	{
		fw_ptr[ctr] = &wavname[ctr][0];
		OpenWavName(ctr);		
	}

	printf("Creating OCH files from %d WAV files. . .\n", fctr_wav);

	strcpy(wavtoochdir, wavpathname);
	strcat(wavtoochdir, "/");
	strcat(wavtoochdir, "wav2och");

   if( mkdir(wavtoochdir, 0777) == 0 )
   {
	   printf("\n");
	   printf("     New folder successfully created\n");
	   printf("     New folder for OCH files: %s\n", wavtoochdir);
   }
   else
   {
	  printf("\n");
      printf("     Path to OCH files: %s\n", wavtoochdir);
   }

	for(ctr=0; ctr<fctr_och; ctr++)						// create header for och
	{
		f_ptr[ctr] = &ochname[ctr][0];
		createOchName(ctr);
		Writeheader(ctr);
	}

/*********************************** main process *********************************************/

	ctr=0;			// upto (fctr_wav-1)
	ochch_ctr=0;
	och_ctr=0;

	nread=0;
	nread2=0;
	printf("     Writing OCH files. . .");

	while(1)
	{

		rd_wav_payload(ctr, (nsamp/2));

		ochch_ctr = ctr % 4;
		transpose_pkt((nsamp/2), ochch_ctr);

		nread++;
		if (16==nread)
		{
			nread = 0;
		}

		if((nread % 4) == 0)
		{
			och_ctr = nread2 % fctr_och;
			wr_audio(och_ctr, (nsamp/2));	
			nread2++;
			if (4==nread2)
			{
				nread2 = 0;
			}
		}

		indicator = getc(wav_stream[ctr]);
		ungetc(indicator, wav_stream[ctr]);

		if((ctr == (fctr_wav-1))&&(indicator == EOF))
			break;

		ctr++;
		ctr %= fctr_wav;


	}
 		printf("Done!\n");
		printf("     %d OCH files written.\n", fctr_och);
	/*************************************************************************************************/

	for(i=0; i<fctr_och; i++)					// close och files
		fclose(mult_stream[i]);

	for(i=0; i<fctr_wav; i++)					// close wave files
		fclose(wav_stream[i]);

	return 0;
}


void OpenWavName(unsigned short ctr)
{
	memset(wavname[ctr], 0, _MAX_PATH);
	itoa( ctr, wavcnt, 10 );

	strcpy(wavname[ctr], wavpathname);
	strcat(wavname[ctr], "/");
	strcat(wavname[ctr], wname);
	strcat(wavname[ctr], wavcnt);
	strcat(wavname[ctr], wavsuff);

	if((wav_stream[ctr] = fopen(fw_ptr[ctr], "rb" )) != NULL )	// must set the name of first file to read
 	{
		fseek(wav_stream[ctr], 44, SEEK_SET);
	
	}else
 	{
		printf("Required WAV file not found or could not be opened. Check number of WAV files.\n");
		printf("The number of WAV files must match 16 files.\n"); 
		exit(1);
	}
}

void createOchName(unsigned short ctr)
{

	memset(ochname[ctr], 0, _MAX_PATH);
	itoa( ctr, filecnt, 10 );

	strcpy(ochname[ctr], wavpathname);
	strcat(ochname[ctr], "/");
	strcat(ochname[ctr], "wav2och");
	strcat(ochname[ctr], "/");
	strcpy(ochmask, name);
	strcat(ochmask, filecnt);
	strcat(ochname[ctr], ochmask);
	strcat(ochname[ctr], suffix);

	if((mult_stream[ctr] = fopen(f_ptr[ctr], "wb" )) != NULL )
	{
	}else
	{
		printf("\n");
		printf( "Could not write file.\n" );
		exit(1);
	}
}


void Writeheader(unsigned short ctr)
{
		if((mult_stream[ctr] = fopen(f_ptr[ctr], "ab" )) != NULL )
		{
			version = 0x20000000;
			fseek(mult_stream[ctr], 0, SEEK_SET);
			fwrite(&version, sizeof(unsigned int), 1, mult_stream[ctr]);
	   
			fseek(mult_stream[ctr], 4, SEEK_SET);
			fwrite(&hexrate, sizeof(unsigned int), 1, mult_stream[ctr]);

			fseek(mult_stream[ctr], 8, SEEK_SET);	
			fwrite(&och_pktsize_resv, sizeof(unsigned short),1, mult_stream[ctr]);		// reserved (och payload size)

			fseek(mult_stream[ctr], 10, SEEK_SET);
			fwrite(&plnum_resv, sizeof(unsigned short), 1, mult_stream[ctr]);
		
			fseek(mult_stream[ctr], 12, SEEK_SET);
			fwrite(&resv, sizeof(unsigned int), 1, mult_stream[ctr]);
		}else
		{
			printf("\n");
			printf( "Could not write file.\n" );
			exit(1);
		}
}


void rd_wav_payload(unsigned short ctr, unsigned short nsamp_2)
{
	fseek(wav_stream[ctr], 0, SEEK_CUR);

	for(i=0; i<nsamp_2; i++)
	{
			fread(&wav_table[i], sizeof(unsigned short), 1, wav_stream[ctr]);
	}
}


void transpose_pkt(unsigned short nsamp_2, unsigned short ochch_ctr)
{
	unsigned short m=0;

	for (i=0; i<nsamp_2; i++)
	{
		tr_data[ochch_ctr][i] = wav_table[m];
		m++;
	}

}

void wr_audio(unsigned short och_ctr, unsigned short nsamp_2)
{
	unsigned short v,x;

		fseek(mult_stream[och_ctr], 0, SEEK_CUR);

		for(v=0; v<OCHCHAN; v++)
		{
			for(x=0; x<nsamp_2; x++)
			{
				fwrite(&tr_data[v][x], sizeof(unsigned short),1, mult_stream[och_ctr]);
			}
		}
}


void HelpScreen()
{
	printf("Creates OCH files from WAV files\n");
	printf("\n");
	printf("WAVTOOCH [SOURCE]\n");
	printf("\n");
	printf("SOURCE       Location of WAV files.\n");
}

